from __future__ import unicode_literals

from django.apps import apps

from evotecore.utils import assign_bulk_perms
from evote.celery import app

from .registers import recipient_register
from .mailings import MessageMail


@app.task(ignore_result=True)
def send_to_recipients(to_send_messageid, recipient_uids):
    """
    TODO: check if who is sending the message have permission for the selected
    recipients
    """
    Message = apps.get_model("evotemessages.Message")  # NOQA
    MessageContact = apps.get_model("evotemessages.MessageContact")  # NOQA
    total_sent = 0
    for recipient_uid in recipient_uids:
        recipient_class, pk = recipient_register.get(recipient_uid)
        recipient = recipient_class()
        obj = recipient.get_object(pk)
        user_ids = recipient.get_user_ids(obj)
        group_object = recipient.get_group_object(obj)
        contacts = MessageContact.objects.get_or_create_contacts(
            user_ids, recipient.namespace)
        total_sent += Message.objects.send_messages(
            to_send_messageid, contacts, group_object, recipient_uid)
    update_messages_permissions.si(to_send_messageid).delay()
    return total_sent


@app.task(ignore_result=True)
def send_to_contacts(to_send_messageid, contact_ids):
    """
    TODO: check if who is sending the message have permission for the selected
    recipients
    """
    Message = apps.get_model("evotemessages.Message")  # NOQA
    MessageContact = apps.get_model("evotemessages.MessageContact")  # NOQA
    message = Message.objects.get(pk=to_send_messageid)
    contacts = MessageContact.objects.filter(pk__in=contact_ids)
    total_sent = Message.objects.send_messages(
        to_send_messageid, contacts, message.content_object)
    update_messages_permissions.si(to_send_messageid).delay()
    return total_sent


@app.task(ignore_result=True)
def update_messages_permissions(sended_messageid):
    Message = apps.get_model("evotemessages.Message")  # NOQA
    messages = Message.objects.select_related('recipient')\
        .filter(parent_id=sended_messageid)
    perms = []
    for message in messages:
        perms.append({
            'object_pk': message.pk,
            'user_id': message.recipient.user_id
        })
    assign_bulk_perms('evotemessages.view_message', perms)
    for message in messages:
        MessageMail(message).send(message.recipient)
    return len(perms)
